#ifndef INC_DSIDEBAR_INL
#define INC_DSIDEBAR_INL

namespace ds {

#ifdef DSLIB
#define DSINLINE 
#else
#define DSINLINE inline
#endif

DSINLINE bool variant2bool(VARIANT_BOOL var)
{
    return var==VARIANT_TRUE;
}

DSINLINE VARIANT_BOOL bool2variant(bool var)
{
    return var?VARIANT_TRUE:VARIANT_FALSE;
}

DSINLINE std::string bstr2str(const CComBSTR& text)
{
    USES_CONVERSION;
    return text?W2T(text):"";
}

DSINLINE std::wstring bstr2wstr(const CComBSTR& text)
{
    return std::wstring(LPWSTR(text?text:L""));
}

DSINLINE CTranslator::CTranslator(CComPtr<ITranslator> translator)
:m_ptr(translator)
{
}

DSINLINE std::string CTranslator::TranslateCommand(const std::string& text) const
{
    CComBSTR retVal;
    CHECKCALL(m_ptr->TranslateCommand(CComBSTR(text.c_str()),&retVal));
    return bstr2str(retVal);
}

DSINLINE void CTranslator::TranslateMenu(HMENU hMenu) const
{
    CHECKCALL(m_ptr->TranslateMenu(hMenu));
}

DSINLINE void CTranslator::TranslateDialog(HWND hDlg, const std::string& name) const
{
    CHECKCALL(m_ptr->TranslateDialog((int)hDlg,CComBSTR(name.c_str())));
}

DSINLINE std::string CTranslator::GetDialogCaption(const std::string& name) const
{
    CComBSTR retVal;
    CHECKCALL(m_ptr->GetDialogCaption(CComBSTR(name.c_str()),&retVal));
    return bstr2str(retVal);
}

DSINLINE std::string CTranslator::date2string(const SYSTEMTIME& time, const std::string& format) const
{
    CComBSTR retVal;
    CHECKCALL(m_ptr->Date2string(&time,CComBSTR(format.c_str()),0,&retVal));
    return bstr2str(retVal);
}

DSINLINE std::string CTranslator::date2string(const SYSTEMTIME& time, DWORD flags) const
{
    CComBSTR retVal;
    CHECKCALL(m_ptr->Date2string(&time,CComBSTR(""),flags,&retVal));
    return bstr2str(retVal);
}

DSINLINE std::string CTranslator::msg(int id) const
{
    char szBuffer[256];
    CComBSTR retVal;
    CHECKCALL(m_ptr->Msg(CComBSTR(itoa(id,szBuffer,10)),&retVal));
    return bstr2str(retVal);
}

DSINLINE std::string CTranslator::msg(const char* id) const
{
    CComBSTR retVal;
    CHECKCALL(m_ptr->Msg(CComBSTR(id),&retVal));
    return bstr2str(retVal);
}

DSINLINE std::string CTranslator::msg(const std::string& id) const
{
    CComBSTR retVal;
    CHECKCALL(m_ptr->Msg(CComBSTR(id.c_str()),&retVal));
    return bstr2str(retVal);
}

DSINLINE CMsgBuilder::CMsgBuilder(const std::string& text)
:m_text(text)
{
}

DSINLINE void CMsgBuilder::push_back(const std::string& param)
{
	m_params.push_back(param);
}

DSINLINE std::string CMsgBuilder::str() const
{
    std::string retVal;

    for (size_t i=0;i<m_text.size();++i)
    {
        char c=m_text[i];
        if (c=='%' && m_text[i+1]=='%')
        {
            retVal+=c;
            ++i;
        }
        else if (c=='%')
        {
            std::string number;
            while(++i<m_text.size() && isdigit(m_text[i]))
            {
                number+=m_text[i];
            }
            --i;

            size_t index=atoi(number.c_str())-1;
            assert(index<m_params.size());
			if (index<m_params.size())
			{
				retVal+=m_params[index];
			}
        }
        else
        {
            retVal+=c;
        }
    }

    return retVal;
}


DSINLINE CGraphics::CGraphics(CComPtr<IGraphics> ptr)
:m_ptr(ptr)
{
}

DSINLINE void CGraphics::Release()
{
    m_ptr.Release();
}

DSINLINE void CGraphics::Takeover(HDC hdc)
{
    CHECKCALL(m_ptr->TakeoverHDC(int(hdc)));
}

DSINLINE Gdiplus::Graphics& CGraphics::Get() const
{
    int graphics;
    CHECKCALL(m_ptr->Get(&graphics));
    return *reinterpret_cast<Gdiplus::Graphics*>(graphics);
}

DSINLINE CPanelParent::CPanelParent()
:m_ptr(0)
{
}

DSINLINE CPanelParent::CPanelParent(CComPtr<IPanelParent> parent)
:m_ptr(parent)
{
}

DSINLINE void CPanelParent::ActivatePanel(int panelCookie)
{
    CHECKCALL(m_ptr->ActivatePanel(panelCookie));
}

DSINLINE void CPanelParent::SetCaption(int panelCookie, const std::string& caption)
{
    CHECKCALL(m_ptr->SetCaption(panelCookie,CComBSTR(caption.c_str())));
}

DSINLINE void CPanelParent::DrawBackground(CGraphics graphics, int panelCookie, const RECT& rc)
{
    CHECKCALL(m_ptr->DrawBackground(graphics.m_ptr, panelCookie, &rc));
}

DSINLINE void CPanelParent::DrawBackground(CGraphics graphics, int panelCookie)
{
    CHECKCALL(m_ptr->DrawPanelBackground(graphics.m_ptr, panelCookie));
}

DSINLINE void CPanelParent::DrawControlBackground(CGraphics graphics, int panelCookie, HWND control)
{
    CHECKCALL(m_ptr->DrawControlBackground(graphics.m_ptr, panelCookie, (int)control));
}

DSINLINE void CPanelParent::ArrangePanels()
{
    CHECKCALL(m_ptr->ArrangePanels());
}

DSINLINE void CPanelParent::InvalidatePanel(HWND hwnd, const RECT* rc, BOOL bErase)
{
    CHECKCALL(m_ptr->InvalidatePanel((int)hwnd, rc, bErase));
}

DSINLINE CSkinElement::CSkinElement(CComPtr<ISkinElement> ptr)
:m_ptr(ptr)
{
}

DSINLINE void CSkinElement::DrawText(
		CGraphics  graphics, 
		const RECT& rect, 
		const std::string&  text,
		int* tabs,
		int tabsCount,
		bool rightToLeft)
{
    CHECKCALL(m_ptr->DrawText(
		graphics.m_ptr, 
		&rect, 
		CComBSTR(text.c_str()),
		tabs,
		tabsCount,
		bool2variant(rightToLeft)));
}

DSINLINE void CSkinElement::DrawText(
		CGraphics  graphics, 
		const RECT& rect, 
		const std::wstring&  text,
		int* tabs,
		int tabsCount,
		bool rightToLeft)
{
    CHECKCALL(m_ptr->DrawText(
		graphics.m_ptr, 
		&rect, 
		CComBSTR(text.c_str()),
		tabs,
		tabsCount,
		bool2variant(rightToLeft)));
}

DSINLINE void CSkinElement::DrawMultilineText(
		CGraphics  graphics, 
		const RECT& rect, 
		const std::string&  text,
		bool rightToLeft)
{
    CHECKCALL(m_ptr->DrawMultilineText(
		graphics.m_ptr, 
		&rect, 
		CComBSTR(text.c_str()),
		bool2variant(rightToLeft)));
}

DSINLINE void CSkinElement::DrawMultilineText(
		CGraphics  graphics, 
		const RECT& rect, 
		const std::wstring&  text,
		bool rightToLeft)
{
    CHECKCALL(m_ptr->DrawMultilineText(
		graphics.m_ptr, 
		&rect, 
		CComBSTR(text.c_str()),
		bool2variant(rightToLeft)));
}

DSINLINE bool CSkinElement::IsHollow() const
{
    VARIANT_BOOL retVal;
    CHECKCALL(m_ptr->IsHollow(&retVal));
    return variant2bool(retVal);
}

DSINLINE void CSkinElement::DrawRect(
    CGraphics  graphics, 
    const RECT& rect, 
    const CPanelParent& parent,
    int cookie)
{
    CHECKCALL(m_ptr->DrawRect(
        graphics.m_ptr,
        &rect,
        parent.m_ptr,
        cookie));
}

DSINLINE void CSkinElement::Fill(
    CGraphics graphics, 
    const RECT& rect)
{
    CHECKCALL(m_ptr->Fill(
        graphics.m_ptr,
        &rect,
        &rect));
}

DSINLINE void CSkinElement::Fill(
    CGraphics graphics, 
    const RECT& skinRect, 
    const RECT& targetRect)
{
    CHECKCALL(m_ptr->Fill(
        graphics.m_ptr,
        &skinRect,
        &targetRect));
}

DSINLINE void CSkinElement::Outline(
    CGraphics  graphics, 
    const RECT& rect)
{
    CHECKCALL(m_ptr->Outline(
        graphics.m_ptr,
        &rect));
}

DSINLINE ETextAlign CSkinElement::GetAlign() const
{
    ETextAlign retVal;
    CHECKCALL(m_ptr->GetAlign(&retVal));
    return retVal;
}

DSINLINE ETextVAlign CSkinElement::GetVAlign() const
{
    ETextVAlign retVal;
    CHECKCALL(m_ptr->GetVAlign(&retVal));
    return retVal;
}

DSINLINE Gdiplus::Color CSkinElement::GetOutlineColor() const
{
    Gdiplus::ARGB argb;
    CHECKCALL(m_ptr->GetOutlineColor(&argb));
    return Gdiplus::Color(argb);
}

DSINLINE EOutlineStyle CSkinElement::GetOutlineStyle() const
{
    EOutlineStyle retVal;
    CHECKCALL(m_ptr->GetOutlineStyle(&retVal));
    return retVal;
}

DSINLINE int CSkinElement::GetOutlineWidth()  const
{
    int retVal;
    CHECKCALL(m_ptr->GetOutlineWidth(&retVal));
    return retVal;
}

DSINLINE EFillStyle CSkinElement::GetFillStyle() const
{
    EFillStyle retVal;
    CHECKCALL(m_ptr->GetFillStyle(&retVal));
    return retVal;
}

DSINLINE Gdiplus::Color CSkinElement::GetFillColor() const
{
    Gdiplus::ARGB argb;
    CHECKCALL(m_ptr->GetFillColor(&argb));
    return Gdiplus::Color(argb);
}

DSINLINE Gdiplus::Color CSkinElement::GetFillColor2() const
{
    Gdiplus::ARGB argb;
    CHECKCALL(m_ptr->GetFillColor2(&argb));
    return Gdiplus::Color(argb);
}

DSINLINE int CSkinElement::GetMetrics(const std::string&  metrics) const
{
    int retVal;
    CHECKCALL(m_ptr->GetMetrics(CComBSTR(metrics.c_str()),&retVal));
    return retVal;
}

DSINLINE SIZE CSkinElement::GetSize(const std::string&  metrics) const
{
    SIZE sz;
    CHECKCALL(m_ptr->GetSize(CComBSTR(metrics.c_str()),&sz));
    return sz;
}

DSINLINE POINT CSkinElement::GetPoint(const std::string&  metrics) const
{
    POINT pt;
    CHECKCALL(m_ptr->GetPoint(CComBSTR(metrics.c_str()),&pt));
    return pt;
}
	
DSINLINE RECT CSkinElement::GetTextMargin() const
{
    RECT rc;
    CHECKCALL(m_ptr->GetTextMargin(&rc));
    return rc;
}

DSINLINE SIZE CSkinElement::GetImageSize() const
{
    SIZE sz;
    CHECKCALL(m_ptr->GetImageSize(&sz));
    return sz;
}

DSINLINE std::string CSkinElement::GetImagePath() const
{
    CComBSTR path;
    CHECKCALL(m_ptr->GetImagePath(&path));
    return bstr2str(path);
}

DSINLINE std::wstring CSkinElement::GetFontFamily() const
{
    CComBSTR name;
    CHECKCALL(m_ptr->GetFontFamily(&name));
    return bstr2wstr(name);
}

DSINLINE int CSkinElement::GetFontSize() const
{
    int size;
    CHECKCALL(m_ptr->GetFontSize(&size));
    return size;
}

DSINLINE EFontStyle CSkinElement::GetFontStyle() const
{
    int style;
    CHECKCALL(m_ptr->GetFontStyle(&style));
    return static_cast<EFontStyle>(style);
}

DSINLINE LOGFONT CSkinElement::GetLogFont() const
{
    LOGFONT lf;
    CHECKCALL(m_ptr->GetLogFont(&lf));
    return lf;
}

DSINLINE Gdiplus::Color CSkinElement::GetTextColor() const
{
    Gdiplus::ARGB argb;
    CHECKCALL(m_ptr->GetTextColor(&argb));
    return Gdiplus::Color(argb); 
}

DSINLINE int CSkinElement::GetFontHeight() const
{
    int height;
    CHECKCALL(m_ptr->GetFontHeight(&height));
    return height;
}

DSINLINE SIZE CSkinElement::GetTextSize(const std::string& text) const
{
    SIZE sz;
    CHECKCALL(m_ptr->GetTextSize(CComBSTR(text.c_str()),&sz));
    return sz;
}

DSINLINE SIZE CSkinElement::GetTextSize(const std::wstring& text) const
{
    SIZE sz;
    CHECKCALL(m_ptr->GetTextSize(CComBSTR(text.c_str()),&sz));
    return sz;
}

DSINLINE int  CSkinElement::GetTextHeight(const std::string& text, int width) const
{
    int retVal;
    CHECKCALL(m_ptr->GetTextHeight(CComBSTR(text.c_str()), width, &retVal));
    return retVal;
}

DSINLINE int  CSkinElement::GetTextHeight(const std::wstring& text, int width) const
{
    int retVal;
    CHECKCALL(m_ptr->GetTextHeight(CComBSTR(text.c_str()), width, &retVal));
    return retVal;
}

DSINLINE void CSkinElement::GetImageMargins(int* left, int* top, int* right, int*  bottom) const
{
    CHECKCALL(m_ptr->GetImageMargins(left,top,right,bottom));
}

DSINLINE RECT CSkinElement::GetImageMargins() const
{
    RECT retVal;
    CHECKCALL(m_ptr->GetImageMargins((int*)&retVal.left,(int*)&retVal.top,(int*)&retVal.right,(int*)&retVal.bottom));
    return retVal;
}

DSINLINE RECT CSkinElement::GetImageMarginsR() const
{
    return GetImageMargins();
}

DSINLINE bool CSkinElement::IsDefined() const
{
    VARIANT_BOOL retVal;
    CHECKCALL(m_ptr->IsDefined(&retVal));
    return variant2bool(retVal);
}

DSINLINE CSkinElementCreate::CSkinElementCreate(CComPtr<ISkinElementCreate> ptr)
:m_ptr(ptr)
{
}

DSINLINE void CSkinElementCreate::SetOutlineStyle(EOutlineStyle outline)
{
    CHECKCALL(m_ptr->SetOutlineStyle(outline));
}

DSINLINE void CSkinElementCreate::SetOutlineColor(Gdiplus::Color color)
{
    CHECKCALL(m_ptr->SetOutlineColor(color.GetValue()));
}

DSINLINE void CSkinElementCreate::SetOutlineWidth(int width)
{
    CHECKCALL(m_ptr->SetOutlineWidth(width));
}

DSINLINE void CSkinElementCreate::SetFillStyle(EFillStyle fill)
{
    CHECKCALL(m_ptr->SetFillStyle(fill));
}

DSINLINE void CSkinElementCreate::SetFillColor(Gdiplus::Color color)
{
    CHECKCALL(m_ptr->SetFillColor(color.GetValue()));
}

DSINLINE void CSkinElementCreate::SetFillColor2(Gdiplus::Color color)
{
    CHECKCALL(m_ptr->SetFillColor2(color.GetValue()));
}

DSINLINE void CSkinElementCreate::SetImagePath(const std::string& path)
{
    CHECKCALL(m_ptr->SetImagePath(CComBSTR(path.c_str())));
}

DSINLINE void CSkinElementCreate::SetImagePath(const std::wstring& path)
{
    CHECKCALL(m_ptr->SetImagePath(CComBSTR(path.c_str())));
}

DSINLINE void CSkinElementCreate::SetImagePathA(const std::string& path)
{
    CHECKCALL(m_ptr->SetImagePath(CComBSTR(path.c_str())));
}


DSINLINE void CSkinElementCreate::SetImageMargins(int left, int top, int right, int bottom)
{
    CHECKCALL(m_ptr->SetImageMargins(left, top, right, bottom));
}

DSINLINE void CSkinElementCreate::SetImageMargins(const RECT& margins)
{
    CHECKCALL(
        m_ptr->SetImageMargins(
            margins.left, margins.top, margins.right, margins.bottom));
}

DSINLINE void CSkinElementCreate::SetImageMarginsR(const RECT& margins)
{
    CHECKCALL(
        m_ptr->SetImageMargins(
            margins.left, margins.top, margins.right, margins.bottom));
}

DSINLINE void CSkinElementCreate::SetFontFamily(const std::string& name )
{
    CHECKCALL(m_ptr->SetFontFamily(CComBSTR(name.c_str())));
}

DSINLINE void CSkinElementCreate::SetFontFamily(const std::wstring& name )
{
    CHECKCALL(m_ptr->SetFontFamily(CComBSTR(name.c_str())));
}

DSINLINE void CSkinElementCreate::SetFontFamilyA(const std::string& name )
{
    CHECKCALL(m_ptr->SetFontFamily(CComBSTR(name.c_str())));
}

DSINLINE void CSkinElementCreate::SetFontFamilyW(const std::wstring& name )
{
    CHECKCALL(m_ptr->SetFontFamily(CComBSTR(name.c_str())));
}

DSINLINE void CSkinElementCreate::SetFontSize(int size )
{
    CHECKCALL(m_ptr->SetFontSize(size));
}

DSINLINE void CSkinElementCreate::SetFontStyle(EFontStyle style)
{
    CHECKCALL(m_ptr->SetFontStyle(style));
}

DSINLINE void CSkinElementCreate::SetTextColor(Gdiplus::Color color)
{
    CHECKCALL(m_ptr->SetTextColor(color.GetValue()));
}

DSINLINE void CSkinElementCreate::SetAlign(ETextAlign align)
{
    CHECKCALL(m_ptr->SetAlign(align));
}

DSINLINE void CSkinElementCreate::SetVAlign(enum ETextVAlign align)
{
    CHECKCALL(m_ptr->SetVAlign(align));
}

DSINLINE CSkinManager::CSkinManager(CComPtr<ISkinManager> ptr)
:m_ptr(ptr)
{
}

DSINLINE CSkinElement CSkinManager::Get(int element, ESkinState state) const
{
    CComPtr<ISkinElement> retVal;
    CHECKCALL(m_ptr->Get(element,state,&retVal));
    return CSkinElement(retVal);
}

DSINLINE CSkinElement CSkinManager::GetCustom(const std::string& elementId, ESkinState state) const
{
    CComPtr<ISkinElement> retVal;
    CHECKCALL(m_ptr->GetCustom(CComBSTR(elementId.c_str()),state,&retVal));
    return CSkinElement(retVal);
}

DSINLINE int CSkinManager::Name2Id(const std::string& elementName) const
{
    int elementId;
    CHECKCALL(m_ptr->Name2Id(CComBSTR(elementName.c_str()),&elementId));
    return elementId;
}

DSINLINE CSkinElementCreate CSkinManager::Create(int elementId, ESkinState state)
{
    CComPtr<ISkinElementCreate> ptr;
    CHECKCALL(m_ptr->Create(elementId,state,&ptr));
    return CSkinElementCreate(ptr);
}

DSINLINE int CSkinManager::Insert(
        CSkinElementCreate normal, 
        CSkinElementCreate hover, 
        CSkinElementCreate pressed,
        CSkinElementCreate undocked)
{
    int id;
    CHECKCALL(m_ptr->Insert(normal.m_ptr,hover.m_ptr,pressed.m_ptr,undocked.m_ptr,&id));
    return id;
}


DSINLINE CAlertManager::CAlertManager(CComPtr<IAlertManager> ptr)
:m_ptr(ptr)
{
}

DSINLINE void CAlertManager::ShowAlert(HWND hwnd, IAlert* alert)
{
    CHECKCALL(m_ptr->ShowAlert((int)hwnd,alert));
}

DSINLINE CUpdateUI::CUpdateUI(CComPtr<IUpdateUI> ptr)
:m_ptr(ptr)
{
}
    
DSINLINE void CUpdateUI::Remove()
{
    CHECKCALL(m_ptr->Remove());
}

DSINLINE void CUpdateUI::SetEnabled(bool enable)
{
    CHECKCALL(m_ptr->SetEnabled(bool2variant(enable)));
}

DSINLINE void CUpdateUI::SetChecked(bool checked)
{
    CHECKCALL(m_ptr->SetChecked(bool2variant(checked)));
}

DSINLINE CCommandManager::CCommandManager(CComPtr<ICommandManager> ptr)
:m_ptr(ptr)
{
}

DSINLINE int CCommandManager::RegisterCommandTarget(const std::string& name, ICommandTarget* pTarget)
{
    int cookie;
    CHECKCALL(m_ptr->RegisterCommandTarget(CComBSTR(name.c_str()), pTarget, &cookie));
    return cookie;
}

DSINLINE void CCommandManager::UnregisterCommandTarget(int cookie)
{
    CHECKCALL(m_ptr->UnregisterCommandTarget(cookie));
}

DSINLINE void CCommandManager::AppendMenu(HMENU hMenu, const std::string& name)
{
    CHECKCALL(m_ptr->AppendMenu(hMenu, CComBSTR(name.c_str())));
}

DSINLINE void CCommandManager::UpdateMenu(ICommandTarget* pGroup, ICommandTarget* pPanel, HMENU hMenu)
{
    CHECKCALL(m_ptr->UpdateMenu(pGroup, pPanel, hMenu));
}

DSINLINE void CCommandManager::SetupToolbar(HWND hwndToolbar, const std::string& name)
{
    CHECKCALL(m_ptr->SetupToolbar((int)hwndToolbar, CComBSTR(name.c_str())));
}

DSINLINE void CCommandManager::UpdateToolbar(ICommandTarget* pGroup, ICommandTarget* pPanel, HWND hwndToolbar)
{
    CHECKCALL(m_ptr->UpdateToolbar(pGroup, pPanel, (int)hwndToolbar));
}

DSINLINE void CCommandManager::DispatchCommand(ICommandTarget* pGroup, ICommandTarget* pPanel, int cmdId)
{
    CHECKCALL(m_ptr->DispatchCommand(pGroup, pPanel, cmdId))
}

DSINLINE void CCommandManager::DispatchCommandUpdate(
	bool* enabled, 
	bool* checked, 
	ICommandTarget* pGroup, 
	ICommandTarget* pPanel, 
	int cmdId)
{
    VARIANT_BOOL venabled,vchecked;
    CHECKCALL(m_ptr->DispatchCommandUpdate(&venabled,&vchecked,pGroup,pPanel,cmdId));
    *enabled=variant2bool(venabled);
    *checked=variant2bool(vchecked);
}

DSINLINE int CCommandManager::GetCommandId(const std::string& cmdName)
{
    int cmd;
    CHECKCALL(m_ptr->GetCommandId(CComBSTR(cmdName.c_str()),&cmd));
    return cmd;
}

DSINLINE int CCommandManager::GetCommandImage(const std::string& cmdName)
{
    int image;
    CHECKCALL(m_ptr->GetCommandImage(CComBSTR(cmdName.c_str()),&image));
    return image;
}

DSINLINE std::string CCommandManager::GetCommandName(int id)
{
    CComBSTR name;
    CHECKCALL(m_ptr->GetCommandName(id,&name));
    return bstr2str(name);
}

DSINLINE HBITMAP CCommandManager::GetImages()
{
    HBITMAP bitmap=NULL;
    CHECKCALL(m_ptr->GetImages(&bitmap));
    return bitmap;
}

DSINLINE int CCommandManager::GetToolbarOffset(const std::string& toolbarId)
{
    int retVal=-1;
    CHECKCALL(m_ptr->GetToolbarOffset(CComBSTR(toolbarId.c_str()),&retVal));
    return retVal;
}

DSINLINE bool CCommandTargetBase::onPrivateCommand(int id)
{
    return false;
}

DSINLINE bool CCommandTargetBase::onUpdatePrivateCommand(int id, CUpdateUI* pCmdUI)
{
    return false;
}

DSINLINE bool CCommandTargetBase::getPrivateCommandImage(int id, int* image)
{
    return false;
}

DSINLINE STDMETHODIMP CCommandTargetChain::OnCommand(BSTR cmdName)
{
    return chain()->onCommand(bstr2str(cmdName))?S_OK:E_FAIL;
}
DSINLINE STDMETHODIMP CCommandTargetChain::OnUpdateCommand(BSTR cmdName, IUpdateUI* pCmdUI)
{
    CUpdateUI wrapper(pCmdUI);
    return chain()->onUpdateCommand(bstr2str(cmdName),&wrapper)?S_OK:E_FAIL;
}
DSINLINE STDMETHODIMP CCommandTargetChain::OnPrivateCommand(int id)
{
    return chain()->onPrivateCommand(id)?S_OK:E_FAIL;
}
DSINLINE STDMETHODIMP CCommandTargetChain::OnUpdatePrivateCommand(int id, IUpdateUI* pCmdUI)
{
    CUpdateUI wrapper(pCmdUI);
    return chain()->onUpdatePrivateCommand(id, &wrapper)?S_OK:E_FAIL;
}
DSINLINE STDMETHODIMP CCommandTargetChain::GetPrivateCommandImage(int id, int* image)
{
    return chain()->getPrivateCommandImage(id,image)?S_OK:E_FAIL;
}

DSINLINE CGlobalSettings::CGlobalSettings(CComPtr<IGlobalSettings> ptr)
:m_ptr(ptr)
{
}

DSINLINE int CGlobalSettings::GetInt(EGlobalParam param) const
{
    int retVal;
    CHECKCALL(m_ptr->GetInt(param,&retVal));
    return retVal;
}

DSINLINE std::string CGlobalSettings::GetString(EGlobalParam param) const
{
    CComBSTR retVal;
    CHECKCALL(m_ptr->GetString(param,&retVal));
    return bstr2str(retVal);
}

DSINLINE std::string CGlobalSettings::GetCodedString(EGlobalParam param) const
{
    CComBSTR retVal;
    CHECKCALL(m_ptr->GetCodedString(param,&retVal));
    return bstr2str(retVal);
}


DSINLINE void CGlobalSettings::SetInt(enum EGlobalParam param, int val)
{
    CHECKCALL(m_ptr->SetInt(param,val));
}

DSINLINE void CGlobalSettings::SetString(enum EGlobalParam param, const std::string& str)
{
    CHECKCALL(m_ptr->SetString(param,CComBSTR(str.c_str())));
}

DSINLINE void CGlobalSettings::SetCodedString(EGlobalParam param, const std::string& str)
{
    CHECKCALL(m_ptr->SetCodedString(param,CComBSTR(str.c_str())));
}

DSINLINE CXmlNode::CXmlNode()
{
}

DSINLINE CXmlNode::CXmlNode(CComPtr<IXmlNode> ptr)
:m_ptr(ptr)
{
}

DSINLINE std::string CXmlNode::GetNameA() const
{
    CComBSTR name;
    CHECKCALL(m_ptr->GetName(&name));
    return bstr2str(name);
}

DSINLINE std::wstring CXmlNode::GetNameW() const
{
    CComBSTR name;
    CHECKCALL(m_ptr->GetName(&name));
    return bstr2wstr(name);
}

DSINLINE std::string CXmlNode::GetBodyA() const
{
    CComBSTR body;
    CHECKCALL(m_ptr->GetBody(&body));
    return bstr2str(body);
}

DSINLINE std::wstring CXmlNode::GetBodyW() const
{
    CComBSTR body;
    CHECKCALL(m_ptr->GetBody(&body));
    return bstr2wstr(body);
}

DSINLINE bool CXmlNode::GetAttribute(std::string* retVal, const std::string& name) const
{
    CComBSTR val;
    if (SUCCEEDED(m_ptr->GetAttribute(CComBSTR(name.c_str()),&val)))
    {
        *retVal=bstr2str(val);
        return true;
    }
    else
    {
        return false;
    }
}

DSINLINE bool CXmlNode::GetAttribute(std::wstring* retVal, const std::string& name) const
{
    CComBSTR val;
    if (SUCCEEDED(m_ptr->GetAttribute(CComBSTR(name.c_str()),&val)))
    {
        *retVal=bstr2wstr(val);
        return true;
    }
    else
    {
        return false;
    }
}

DSINLINE std::string CXmlNode::GetAttribute(const std::string& name,  const std::string& defValue) const
{
    CComBSTR val;
    if (SUCCEEDED(m_ptr->GetAttribute(CComBSTR(name.c_str()),&val)))
    {
        return bstr2str(val);
    }
    else
    {
        return defValue;
    }
}

DSINLINE std::string CXmlNode::GetObligAttributeA(const std::string& name) const
{
    CComBSTR val;
    if (SUCCEEDED(m_ptr->GetAttribute(CComBSTR(name.c_str()),&val)))
    {
        return bstr2str(val);
    }
    else
    {
        throw std::runtime_error(std::string("missing attribute ")+name);
    }
}

DSINLINE std::wstring CXmlNode::GetObligAttributeW(const std::string& name) const
{
    CComBSTR val;
    if (SUCCEEDED(m_ptr->GetAttribute(CComBSTR(name.c_str()),&val)))
    {
        return bstr2wstr(val);
    }
    else
    {
        throw std::runtime_error("missing attribute "+name);
    }
}

DSINLINE std::wstring CXmlNode::GetAttribute(const std::wstring& name,  const std::wstring& defValue) const
{
    CComBSTR val;
    if (SUCCEEDED(m_ptr->GetAttribute(CComBSTR(name.c_str()),&val)))
    {
        return bstr2wstr(val);
    }
    else
    {
        return defValue;
    }
}

DSINLINE CXmlNode CXmlNode::GetNode(const std::string& path) const
{
    CComPtr<IXmlNode> node;
    m_ptr->GetNode(CComBSTR(path.c_str()),&node);
    return CXmlNode(node);
}

DSINLINE CXmlNode::iterator::iterator()
:m_ptr(0),m_current(0)
{
}

DSINLINE CXmlNode::iterator::iterator(CComPtr<IXmlNode> ptr)
:m_ptr(ptr),m_current(0)
{
}

DSINLINE CXmlNode::iterator::iterator(const CXmlNode::iterator& other)
{
    if (this!=&other)
    {
	    m_ptr = other.m_ptr;
	    m_current = other.m_current;
    }
}

DSINLINE CXmlNode::iterator& CXmlNode::iterator::operator=(const CXmlNode::iterator & other)
{
    if (this!=&other)
    {
	    m_ptr = other.m_ptr;
	    m_current = other.m_current;
    }
	return *this;
}

DSINLINE CXmlNode CXmlNode::iterator::operator*() const
{ 
    CComPtr<IXmlNode> node;
    CHECKCALL(m_ptr->GetChild(m_current,&node));
    return CXmlNode(node); 
}

DSINLINE CXmlNode::iterator & CXmlNode::iterator::operator++()
{
    next();
	return *this;
}

DSINLINE CXmlNode::iterator CXmlNode::iterator::operator++(int) 
{	
	iterator pom(*this);			
    next();
	return pom;	
}

DSINLINE void CXmlNode::iterator::next()
{
    int size;
    CHECKCALL(m_ptr->GetChildCount(&size));
    if (++m_current>=size)
    {
        m_ptr.Release();
        m_current=0;
    }
}

DSINLINE bool CXmlNode::iterator::operator==(const CXmlNode::iterator & it) const
{ 
    return m_ptr==it.m_ptr && m_current==it.m_current; 
}

DSINLINE bool CXmlNode::iterator::operator!=(const CXmlNode::iterator & it) const
{
    return m_ptr!=it.m_ptr || m_current!=it.m_current; 
}

/*-----------------------------------------------------------------------------
-----------------------------------------------------------------------------*/
DSINLINE CXmlNode::iterator CXmlNode::begin() const
{
    int size;
    CHECKCALL(m_ptr->GetChildCount(&size));
    if (size)
    {
        return iterator(m_ptr);
    }
    else
    {
        return iterator(0);
    }
}

DSINLINE CXmlNode::iterator CXmlNode::end() const
{
    return iterator(0);
}

DSINLINE CPanelConfig::CPanelConfig()
:m_ptr(0)
{
}

DSINLINE CPanelConfig::CPanelConfig(CComPtr<IPanelConfig> ptr)
:m_ptr(ptr)
{
}

DSINLINE bool CPanelConfig::AutoFit() const
{
    VARIANT_BOOL retVal;
    CHECKCALL(m_ptr->AutoFit(&retVal));
    return variant2bool(retVal);
}

DSINLINE void CPanelConfig::SetAutoFit(bool autofit)
{
    CHECKCALL(m_ptr->SetAutoFit(bool2variant(autofit)));
}

DSINLINE SIZE CPanelConfig::GetDetailsSize() const
{
    SIZE retVal;
    CHECKCALL(m_ptr->GetDetailsSize(&retVal));
    return retVal;
}

DSINLINE void CPanelConfig::SetDetailsSize(const SIZE& size)
{
    CHECKCALL(m_ptr->SetDetailsSize(size));
}

DSINLINE bool CPanelConfig::AutoStretch() const
{
    VARIANT_BOOL retVal;
    CHECKCALL(m_ptr->AutoStretch(&retVal));
    return variant2bool(retVal);
}

DSINLINE void CPanelConfig::SetAutoStretch(bool autoexpand)
{
    CHECKCALL(m_ptr->SetAutoStretch(bool2variant(autoexpand)));
}

DSINLINE bool CPanelConfig::IsExpanded() const
{
    VARIANT_BOOL retVal;
    CHECKCALL(m_ptr->IsExpanded(&retVal));
    return variant2bool(retVal);
}

DSINLINE void CPanelConfig::Expand(bool expand)
{
    CHECKCALL(m_ptr->Expand(bool2variant(expand)));
}

DSINLINE int CPanelConfig::GetHeight(int width) const
{
    int retVal;
    CHECKCALL(m_ptr->GetHeight(&retVal));
    return retVal;
}

DSINLINE void CPanelConfig::SetHeight(int height)
{
    CHECKCALL(m_ptr->SetHeight(height));
}

DSINLINE void CPanelConfig::ShowCaption(bool showcaption)
{
    CHECKCALL(m_ptr->ShowCaption(bool2variant(showcaption)));
}

DSINLINE bool CPanelConfig::IsCaptionVisible() const
{
    VARIANT_BOOL retVal;
    CHECKCALL(m_ptr->IsCaptionVisible(&retVal));
    return variant2bool(retVal);
}

DSINLINE bool CPanelConfig::NeedUpdate()
{
    VARIANT_BOOL retVal;
    CHECKCALL(m_ptr->NeedUpdate(&retVal));
    return variant2bool(retVal);
}

DSINLINE bool CPanelConfig::IsUndocked() const
{
    VARIANT_BOOL retVal;
    CHECKCALL(m_ptr->IsUndocked(&retVal));
    return variant2bool(retVal);
}

DSINLINE bool CPanelConfig::SupportsOffScreen() const
{
    VARIANT_BOOL retVal;
    CHECKCALL(m_ptr->SupportsOffScreen(&retVal));
    return variant2bool(retVal);
}

DSINLINE CXmlNode CPanelConfig::GetConfigRoot()
{
    CComPtr<IXmlNode> retVal;
    CHECKCALL(m_ptr->GetConfigRoot(&retVal));
    return CXmlNode(retVal);
}

DSINLINE CXmlBuilder::CXmlBuilder(CComPtr<IXmlBuilder> ptr)
:m_ptr(ptr)
{
}

DSINLINE CXmlBuilder CXmlBuilder::AddChild(const std::string& tag)
{
    CComPtr<IXmlBuilder> child;
    CHECKCALL(m_ptr->AddChild(CComBSTR(tag.c_str()),&child));
    return CXmlBuilder(child);
}

//-----------------------------------------------------------------------------
// CControl routines
//-----------------------------------------------------------------------------

DSINLINE CControl::CControl()
{
}

DSINLINE CControl::CControl(CComPtr<IUnknown> ptr)
{
    CComQIPtr<IControl,&IID_IControl> ctrl(ptr);
    m_ptr = ctrl;
    assert(m_ptr != 0);
}

DSINLINE CControl::CControl(CComPtr<IControl> ptr):m_ptr(ptr)
{
}

DSINLINE void CControl::Init(const std::string& idctrl)
{
    CHECKCALL(m_ptr->Init(CComBSTR(idctrl.c_str())));
}

DSINLINE void CControl::Create(HWND hwndParent, const bool visible)
{
    CHECKCALL(m_ptr->Create(reinterpret_cast<int>(hwndParent),bool2variant(visible)));
}

DSINLINE std::string CControl::GetClass()
{
    CComBSTR id;
    CHECKCALL(m_ptr->GetClass(&id));
    return bstr2str(id);
}

DSINLINE std::string CControl::GetControlId()
{
    CComBSTR id;
    CHECKCALL(m_ptr->GetControlId(&id));
    return bstr2str(id);
}

DSINLINE void CControl::Close()
{
    CHECKCALL(m_ptr->Close());
}

DSINLINE void CControl::MoveWindow(const int x, const int y, const int width, const int height)
{
    CHECKCALL(m_ptr->MoveWindow(x,y,width,height));
}

DSINLINE void CControl::ShowWindow(const bool show)
{
    CHECKCALL(m_ptr->ShowWindow(bool2variant(show)));
}

DSINLINE void CControl::GetControlRect(RECT* rect)
{
    CHECKCALL(m_ptr->GetControlRect(rect));
}

DSINLINE void CControl::Invalidate()
{
    CHECKCALL(m_ptr->Invalidate());
}

DSINLINE int CControl::GetFitHeightEx(const int width)
{
    int height;
    CHECKCALL(m_ptr->GetFitHeightEx(width,&height));
    return height;
}

DSINLINE int CControl::GetFitWidthEx(const int height)
{
    int width;
    CHECKCALL(m_ptr->GetFitWidthEx(height,&width));
    return width;
}

DSINLINE SIZE CControl::GetFitSize()
{
    SIZE size;
    CHECKCALL(m_ptr->GetFitSize(&size));
    return size;
}

DSINLINE void CControl::SetControlSite(IControlSite* site)
{
    CHECKCALL(m_ptr->SetControlSite(site));
}

DSINLINE void CControl::SetParent(IUnknown* parent)
{
    CHECKCALL(m_ptr->SetParent(parent));
}

DSINLINE void CControl::OnPaintEvent(CGraphics graphics)
{
    CHECKCALL(m_ptr->OnPaintEvent(graphics.m_ptr));
}

DSINLINE void CControl::OnMouseEvent(EMouseEvent event, const bool dbclk, POINT pos)
{
    CHECKCALL(m_ptr->OnMouseEvent(event,bool2variant(dbclk),pos));
}

//-----------------------------------------------------------------------------
// CLayout routines
//-----------------------------------------------------------------------------

DSINLINE CLayout::CLayout()
{
}
    
DSINLINE CLayout::CLayout(CComPtr<ILayout> ptr):m_ptr(ptr)
{
}
    
DSINLINE void CLayout::AddControl(CControl ctrl)
{
    CHECKCALL(m_ptr->AddControl(ctrl.m_ptr));
}

DSINLINE CControl CLayout::GetControl(const std::string& idctrl)
{
    CComPtr<IControl> control;
    CHECKCALL(m_ptr->GetControl(CComBSTR(idctrl.c_str()),&control));
    return CControl(control);
}

DSINLINE void CLayout::SetEventHandlers(CEventHandlersManager eventHandlers)
{
    CHECKCALL(m_ptr->SetEventHandlers(eventHandlers.m_ptr));
}

//-----------------------------------------------------------------------------
// CCanvas routines
//-----------------------------------------------------------------------------

DSINLINE CCanvas::CCanvas()
{
}

DSINLINE CCanvas::CCanvas(CComPtr<ICanvas> ptr):m_ptr(ptr)
{
}

DSINLINE void CCanvas::Init(
    CSidebar sidebar,
    HWND parentHwnd)
{
    CComPtr<IUnknown> raw_sidebar(sidebar.m_ptr);
    CHECKCALL(m_ptr->Init(
        raw_sidebar,
        reinterpret_cast<int>(parentHwnd)));
}

DSINLINE CControl CCanvas::GetControl(const std::string& idctrl)
{
    CComPtr<IControl> control;
    CHECKCALL(m_ptr->GetControl(CComBSTR(idctrl.c_str()),&control));
    return CControl(control);
}

DSINLINE std::string CCanvas::GetEventAction(
    const std::string& idctrl,
    const std::string& event)
{
    CComBSTR id;
    CHECKCALL(
        m_ptr->GetEventAction(
            CComBSTR(idctrl.c_str()),
            CComBSTR(event.c_str()),
            &id));
    return bstr2str(id);
}

DSINLINE void CCanvas::AddEventAction(
    const std::string& idctrl,
    const std::string& event,
    const std::string& action)
{
    CHECKCALL(
        m_ptr->AddEventAction(
            CComBSTR(idctrl.c_str()),
            CComBSTR(event.c_str()),
            CComBSTR(action.c_str())));
}

DSINLINE int CCanvas::GetControlsCount()
{
    int counter;
    CHECKCALL(m_ptr->GetControlsCount(&counter));
    return counter;
}

DSINLINE CControl CCanvas::GetControlByIndex(const int index)
{
    CComPtr<IControl> control;
    CHECKCALL(m_ptr->GetControlByIndex(index,&control));
    return CControl(control);
}

DSINLINE void CCanvas::RegisterControl(CControl ctrl)
{
    CHECKCALL(m_ptr->RegisterControl(ctrl.m_ptr));
}

//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------

DSINLINE STDMETHODIMP CTextOutputParent::OnDrawBackground(ITextOutput* ptr, IGraphics* graphics)
{
    CTextOutput textOutput(ptr);
    OnDrawBackground(textOutput,CGraphics(graphics));
    return S_OK;
}

DSINLINE STDMETHODIMP CTextOutputParent::OnClick(ITextOutput* ptr, VARIANT_BOOL dbclk)
{
    CTextOutput textOutput(ptr);
    OnClick(textOutput,variant2bool(dbclk));
    return S_OK;
}

DSINLINE STDMETHODIMP CTextOutputParent::OnMouseHover(ITextOutput* ptr)
{
    CTextOutput textOutput(ptr);
    OnMouseHover(textOutput);
    return S_OK;
}

DSINLINE STDMETHODIMP CTextOutputParent::OnMouseLeave(ITextOutput* ptr)
{
    CTextOutput textOutput(ptr);
    OnMouseLeave(textOutput);
    return S_OK;
}

DSINLINE STDMETHODIMP CTextOutputParent::OnShowDetails(ITextOutput* ptr)
{
    CTextOutput textOutput(ptr);
    OnShowDetails(textOutput);
    return S_OK;
}

DSINLINE CTextOutput::CTextOutput()
:m_ptr(0)
{
}

DSINLINE CTextOutput::CTextOutput(CComPtr<ITextOutput> ptr)
:m_ptr(ptr)
{
}
	
DSINLINE void CTextOutput::Release()
{
    m_ptr.Release();
}

DSINLINE bool CTextOutput::Init(
	CGlobalSettings settings, 
	CSkinManager skinManager, 
	ITextOutputParent* parent,
    bool supportClick,
    int id)
{
    return SUCCEEDED(m_ptr->Init2(
	    settings.m_ptr, 
	    skinManager.m_ptr, 
	    parent,
        bool2variant(supportClick),
        id));
}

DSINLINE int CTextOutput::GetId() const
{
    int retVal;
    CHECKCALL(m_ptr->GetId(&retVal));
    return retVal;
}

DSINLINE bool CTextOutput::Create(HWND hwndParent, bool visible)
{
    return SUCCEEDED(m_ptr->Create(
	    (int)hwndParent, 
	    bool2variant(visible)));
}

DSINLINE HWND CTextOutput::GetHwnd()
{
    int retVal;
    CHECKCALL(m_ptr->GetHwnd(&retVal));
    return (HWND)retVal;
}

DSINLINE void CTextOutput::Close()
{
    CHECKCALL(m_ptr->Close());
}

DSINLINE void CTextOutput::MoveWindow(int x, int y, int width, int height)
{
    CHECKCALL(m_ptr->MoveWindow(x, y, width, height));
}

DSINLINE void CTextOutput::ShowWindow(bool show)
{
    CHECKCALL(m_ptr->ShowWindow(bool2variant(show)));
}

DSINLINE void CTextOutput::GetControlRect(RECT* rect)
{
    CHECKCALL(m_ptr->GetControlRect(rect));
}

DSINLINE void CTextOutput::SetText(const std::string& text)
{
    CHECKCALL(m_ptr->SetText(CComBSTR(text.c_str())));
}

DSINLINE void CTextOutput::SetSkinElement(int skinElement)
{
    CHECKCALL(m_ptr->SetSkinElement(skinElement));
}

DSINLINE int CTextOutput::GetSkinElement()
{
    int retVal;
    CHECKCALL(m_ptr->GetSkinElement(&retVal));
    return retVal;
}

DSINLINE int CTextOutput::GetFitHeight()
{
    int retVal;
    CHECKCALL(m_ptr->GetFitHeight(&retVal));
    return retVal;
}

DSINLINE int CTextOutput::GetFitHeightEx(int width)
{
    int retVal;
    CHECKCALL(m_ptr->GetFitHeightEx(width,&retVal));
    return retVal;
}

DSINLINE void CTextOutput::TakeoverDetails(CDetailsWnd details)
{
    CHECKCALL(m_ptr->TakeoverDetails(details.m_ptr));
}

DSINLINE void CTextOutput::SetWrap(bool wrap)
{
    CHECKCALL(m_ptr->SetWrap(bool2variant(wrap)));
}

DSINLINE bool CTextOutput::GetWrap()
{
    VARIANT_BOOL retVal;
    CHECKCALL(m_ptr->GetWrap(&retVal));
    return variant2bool(retVal);
}

DSINLINE std::string CTextOutput::GetText() const
{
    CComBSTR retVal;
    CHECKCALL(m_ptr->GetText(&retVal));
    return bstr2str(retVal);

}

DSINLINE void CTextOutput::SetParent(IUnknown* parent)
{
    CHECKCALL(m_ptr->SetParent(parent));
}

DSINLINE void CTextOutput::SetIdentifier(const int identifier)
{
    CHECKCALL(m_ptr->SetIdentifier(identifier));
}

DSINLINE void CTextOutput::SetSupportClick(const bool supportClick)
{
    CHECKCALL(m_ptr->SetSupportClick(bool2variant(supportClick)));
}


DSINLINE CListRow::CListRow(CComPtr<IListRow> ptr)
:m_ptr(ptr)
{
}

DSINLINE void CListRow::Release()
{
    m_ptr.Release();
}

DSINLINE void CListRow::SetImageSkin(int image)
{
    CHECKCALL(m_ptr->SetImageSkin(image));
}

DSINLINE void CListRow::SetTextSkin(int skin)
{
    CHECKCALL(m_ptr->SetTextSkin(skin));
}

DSINLINE void CListRow::SetBackgroundSkin(int skin)
{
    CHECKCALL(m_ptr->SetBackgroundSkin(skin));
}

DSINLINE void CListRow::SetText(const std::string& text)
{
    CHECKCALL(m_ptr->SetText(CComBSTR(text.c_str())));
}

DSINLINE void CListRow::SetText(const std::wstring& text)
{
    CHECKCALL(m_ptr->SetText(CComBSTR(text.c_str())));
}

DSINLINE std::wstring CListRow::GetText()
{
    CComBSTR text;
    CHECKCALL(m_ptr->GetText(&text));
    return bstr2wstr(text);
}

DSINLINE void CListRow::SetDetailsText(const std::string& text)
{
    CHECKCALL(m_ptr->SetDetailsText(CComBSTR(text.c_str())));
}

DSINLINE void CListRow::SetDetailsText(const std::wstring& text)
{
    CHECKCALL(m_ptr->SetDetailsText(CComBSTR(text.c_str())));
}

DSINLINE std::wstring CListRow::GetDetailsText()
{
    CComBSTR text;
    CHECKCALL(m_ptr->GetDetailsText(&text));
    return bstr2wstr(text);
}

DSINLINE int CListRow::GetUserData(int index)
{
    int retVal;
    CHECKCALL(m_ptr->GetUserData(index,&retVal));
    return retVal;
}

DSINLINE void CListRow::SetUserData(int index, int value)
{
    CHECKCALL(m_ptr->SetUserData(index,value));
}

DSINLINE STDMETHODIMP CListOutputParent::OnDrawBackground(IListOutput* ptr, IGraphics* graphics)
{
    CListOutput listOutput(ptr);
    OnDrawBackground(listOutput,CGraphics(graphics));
    return S_OK;
}

DSINLINE STDMETHODIMP CListOutputParent::OnClick(IListOutput* ptr, IListRow* row, VARIANT_BOOL dbclk)
{
    CListOutput listOutput(ptr);
    CListRow listRow(row);
    OnClick(listOutput,&listRow,variant2bool(dbclk));
    return S_OK;
}

DSINLINE STDMETHODIMP CListOutputParent::OnHover(IListOutput* ptr, IListRow* row)
{
    CListOutput listOutput(ptr);
    CListRow listRow(row);
    OnHover(listOutput,&listRow);
    return S_OK;
}

DSINLINE STDMETHODIMP CListOutputParent::OnLeave(IListOutput* ptr, IListRow* row)
{
    CListOutput listOutput(ptr);
    CListRow listRow(row);
    OnLeave(listOutput,&listRow);
    return S_OK;
}

DSINLINE STDMETHODIMP CListOutputParent::OnShowDetails(IListOutput* ptr, IListRow* row)
{
    CListOutput listOutput(ptr);
    CListRow listRow(row);
    OnShowDetails(listOutput,&listRow);
    return S_OK;
}

DSINLINE STDMETHODIMP CListOutputParent::OnScrolled(IListOutput* ptr)
{
    CListOutput listOutput(ptr);
    OnScrolled(listOutput);
    return S_OK;
}

DSINLINE CListOutput::CListOutput()
:m_ptr(0)
{
}

DSINLINE CListOutput::CListOutput(CComPtr<IListOutput> ptr)
:m_ptr(ptr)
{
}

DSINLINE void CListOutput::Release()
{
    m_ptr.Release();
}

DSINLINE bool CListOutput::Init(
		CGlobalSettings settings, 
		CSkinManager skinManager, 
		IListOutputParent* parent,
        bool supportAutoScrolling,
        int id)
{
    return SUCCEEDED(m_ptr->Init2(
	    settings.m_ptr, 
		skinManager.m_ptr, 
		parent,
        bool2variant(supportAutoScrolling),
        id));
}
	
DSINLINE int CListOutput::GetId() const
{
    int retVal;
    CHECKCALL(m_ptr->GetId(&retVal));
    return retVal;
}

DSINLINE bool CListOutput::Create(HWND hwndParent, bool visible)
{
    return SUCCEEDED(m_ptr->Create((int)hwndParent,bool2variant(visible)));
}

DSINLINE HWND CListOutput::GetHwnd() const
{
    int retVal;
    CHECKCALL(m_ptr->GetHwnd(&retVal));
    return (HWND)retVal;
}

DSINLINE void CListOutput::Close()
{
    CHECKCALL(m_ptr->Close());
}

DSINLINE void CListOutput::MoveWindow(int x, int y, int width, int height)
{
    CHECKCALL(m_ptr->MoveWindow(x, y, width, height));
}

DSINLINE void CListOutput::ShowWindow(bool show)
{
    CHECKCALL(m_ptr->ShowWindow(bool2variant(show)));
}

DSINLINE void CListOutput::GetControlRect(RECT* rect)
{
    CHECKCALL(m_ptr->GetControlRect(rect));
}

DSINLINE void CListOutput::Invalidate()
{
    CHECKCALL(m_ptr->Invalidate());
}

DSINLINE void CListOutput::SetSkinElement(int skinElement)
{
    CHECKCALL(m_ptr->SetSkinElement(skinElement));
}

DSINLINE int CListOutput::GetSkinElement() const
{
    int retVal;
    CHECKCALL(m_ptr->GetSkinElement(&retVal));
    return retVal;
}

DSINLINE void CListOutput::ShowRowSeparator(bool showSeparator)
{
    CHECKCALL(m_ptr->ShowRowSeparator(bool2variant(showSeparator)));
}

DSINLINE void CListOutput::WrapText(bool wrap)
{
    CHECKCALL(m_ptr->WrapText(bool2variant(wrap)));
}

DSINLINE void CListOutput::SetRightToLeft(bool rightToLeft)
{
    CHECKCALL(m_ptr->SetRightToLeft(bool2variant(rightToLeft)));
}

DSINLINE int CListOutput::GetFitHeight(int width) const
{
    int retVal;
    CHECKCALL(m_ptr->GetFitHeight(width,&retVal));
    return retVal;
}

DSINLINE CListRow CListOutput::AddRow()
{
    CComPtr<IListRow> retVal;
    CHECKCALL(m_ptr->AddRow(&retVal));
    return CListRow(retVal);
}

DSINLINE CListRow CListOutput::InsertRow(int insertBefore)
{
    CComPtr<IListRow> retVal;
    CHECKCALL(m_ptr->InsertRow(insertBefore,&retVal));
    return CListRow(retVal);
}

DSINLINE CListRow CListOutput::GetRow(size_t index) const
{
    CListRow retVal(0);
    CHECKCALL(m_ptr->GetRow(index,&retVal.m_ptr));
    return retVal;
}

DSINLINE int CListOutput::Size() const
{
    int retVal;
    CHECKCALL(m_ptr->Size(&retVal));
    return retVal;
}

DSINLINE int CListOutput::GetFirstVisible() const
{
    int retVal;
    CHECKCALL(m_ptr->GetFirstVisible(&retVal));
    return retVal;
}

DSINLINE int CListOutput::GetLastVisible() const
{
    int retVal;
    CHECKCALL(m_ptr->GetLastVisible(&retVal));
    return retVal;
}

DSINLINE int CListOutput::GetHoverRow() const
{
    int retVal;
    CHECKCALL(m_ptr->GetHoverRow(&retVal));
    return retVal;
}

DSINLINE void CListOutput::Clear()
{
    CHECKCALL(m_ptr->Clear());
}
	
DSINLINE RECT CListOutput::GetRowRect(CListRow row) const
{
    RECT retVal;
    CHECKCALL(m_ptr->GetRowRect(row.m_ptr,&retVal));
    return retVal;
}

DSINLINE CListRow CListOutput::GetRowFromPoint(const POINT& pt) const
{
    CListRow retVal(0);
    m_ptr->GetRowFromPoint(pt,&retVal.m_ptr);
    return retVal;
}

DSINLINE void CListOutput::ScrollToPrev()
{
    CHECKCALL(m_ptr->ScrollToPrev());
}

DSINLINE void CListOutput::ScrollToRow(size_t row)
{
    CHECKCALL(m_ptr->ScrollToRow(row));
}

DSINLINE void CListOutput::ScrollToNext()
{
    CHECKCALL(m_ptr->ScrollToNext());
}

DSINLINE void CListOutput::SetIndent(int image, int text)
{
    CHECKCALL(m_ptr->SetIndent(image,text));
}

DSINLINE void CListOutput::SetTabs(int* tabs, int tabsCount)
{
    CHECKCALL(m_ptr->SetTabs(tabs,tabsCount));
}

DSINLINE void CListOutput::TakeoverDetails(CDetailsWnd details, CListRow row)
{
    CHECKCALL(m_ptr->TakeoverDetails(details.m_ptr,row.m_ptr));
}

DSINLINE void CListOutput::SetIdentifier(const int identifier)
{
    CHECKCALL(m_ptr->SetIdentifier(identifier));
}

DSINLINE void CListOutput::SetAutoScrolling(const bool supportAutoScrolling)
{
    CHECKCALL(m_ptr->SetAutoScrolling(bool2variant(bool2variant(supportAutoScrolling))));
}

DSINLINE void CListOutput::SetMaxFitRows(const int maxFitRows)
{
    CHECKCALL(m_ptr->SetMaxFitRows(maxFitRows));
}


DSINLINE STDMETHODIMP CMarqueeOutputParent::OnDrawBackground(IMarqueeOutput* ptr, IGraphics* graphics)
{
    CMarqueeOutput marquee(ptr);
    OnDrawBackground(marquee,CGraphics(graphics));
    return S_OK;
}

DSINLINE STDMETHODIMP CMarqueeOutputParent::OnClick(IMarqueeOutput* ptr, VARIANT_BOOL dbclk)
{
    CMarqueeOutput marquee(ptr);
    OnClick(marquee,variant2bool(dbclk));
    return S_OK;
}

DSINLINE STDMETHODIMP CMarqueeOutputParent::OnMouseHover(IMarqueeOutput* ptr)
{
    CMarqueeOutput marquee(ptr);
    OnMouseHover(marquee);
    return S_OK;
}

DSINLINE STDMETHODIMP CMarqueeOutputParent::OnMouseLeave(IMarqueeOutput* ptr)
{
    CMarqueeOutput marquee(ptr);
    OnMouseLeave(marquee);
    return S_OK;
}

DSINLINE STDMETHODIMP CMarqueeOutputParent::OnShowDetails(IMarqueeOutput* ptr)
{
    CMarqueeOutput marquee(ptr);
    OnShowDetails(marquee);
    return S_OK;
}

DSINLINE CMarqueeOutput::CMarqueeOutput()
:m_ptr(0)
{
}

DSINLINE CMarqueeOutput::CMarqueeOutput(CComPtr<IMarqueeOutput> ptr)
:m_ptr(ptr)
{
}
	
DSINLINE void CMarqueeOutput::Release()
{
    m_ptr.Release();
}

DSINLINE bool CMarqueeOutput::Init(
	CGlobalSettings settings, 
	CSkinManager skinManager, 
	IMarqueeOutputParent* parent,
    bool supportClick,
    int id)
{
    return SUCCEEDED(m_ptr->Init2(
	    settings.m_ptr, 
		skinManager.m_ptr, 
		parent,
        bool2variant(supportClick),
        id));
}
	
DSINLINE int CMarqueeOutput::GetId() const
{
    int retVal;
    CHECKCALL(m_ptr->GetId(&retVal));
    return retVal;
}

DSINLINE bool CMarqueeOutput::Create(HWND hwndParent, bool visible)
{
    return SUCCEEDED(m_ptr->Create((int)hwndParent,bool2variant(visible)));
}

DSINLINE HWND CMarqueeOutput::GetHwnd()
{
    int retVal;
    CHECKCALL(m_ptr->GetHwnd(&retVal));
    return (HWND)retVal;
}

DSINLINE void CMarqueeOutput::Close()
{
    CHECKCALL(m_ptr->Close());
}

DSINLINE void CMarqueeOutput::MoveWindow(int x, int y, int width, int height)
{
    CHECKCALL(m_ptr->MoveWindow(x, y, width, height));
}

DSINLINE void CMarqueeOutput::ShowWindow(bool show)
{
    CHECKCALL(m_ptr->ShowWindow(bool2variant(show)));
}

DSINLINE void CMarqueeOutput::GetControlRect(RECT* rect)
{
    CHECKCALL(m_ptr->GetControlRect(rect));
}

DSINLINE void CMarqueeOutput::SetText(const std::string& text)
{
    CHECKCALL(m_ptr->SetText(CComBSTR(text.c_str())));
}

DSINLINE void CMarqueeOutput::SetSkinElement(int skinElement)
{
    CHECKCALL(m_ptr->SetSkinElement(skinElement));
}

DSINLINE int CMarqueeOutput::GetSkinElement()
{
    int retVal;
    CHECKCALL(m_ptr->GetSkinElement(&retVal));
    return retVal;
}

DSINLINE int CMarqueeOutput::GetFitHeight()
{
    int retVal;
    CHECKCALL(m_ptr->GetFitHeight(&retVal));
    return retVal;
}

DSINLINE void CMarqueeOutput::SetSpeed(int frequency, int delta)
{
    CHECKCALL(m_ptr->SetSpeed(frequency, delta));
}

DSINLINE void CMarqueeOutput::TakeoverDetails(CDetailsWnd details)
{
    CHECKCALL(m_ptr->TakeoverDetails(details.m_ptr));
}

DSINLINE void CMarqueeOutput::SetIdentifier(const int identifier)
{
    CHECKCALL(m_ptr->SetIdentifier(identifier));
}

DSINLINE void CMarqueeOutput::SetSupportClick(const bool supportClick)
{
    CHECKCALL(m_ptr->SetSupportClick(bool2variant(supportClick)));
}


DSINLINE CSkinButton::CSkinButton()
:m_ptr(0)
{
}

DSINLINE CSkinButton::CSkinButton(CComPtr<ISkinButton> ptr)
:m_ptr(ptr)
{
}
	
DSINLINE void CSkinButton::Release()
{
    m_ptr.Release();
}

DSINLINE bool CSkinButton::Init(
		CSkinManager skinManager, 
		ISkinButtonParent* parent,
        int normalSkin, 
        int checkedSkin, 
        int commandId)
{
    return SUCCEEDED(m_ptr->Init(
        skinManager.m_ptr,
        parent,
        normalSkin,
        checkedSkin,
        commandId));
}
	
DSINLINE bool CSkinButton::Create(HWND hwndParent, bool visible)
{
    return SUCCEEDED(m_ptr->Create((int)hwndParent,bool2variant(visible)));
}

DSINLINE HWND CSkinButton::GetHwnd()
{
    int retVal;
    CHECKCALL(m_ptr->GetHwnd(&retVal));
    return (HWND)retVal;
}

DSINLINE void CSkinButton::Close()
{
    CHECKCALL(m_ptr->Close());
}

DSINLINE void CSkinButton::MoveWindow(int x, int y, int width, int height)
{
    CHECKCALL(m_ptr->MoveWindow(x, y, width, height));
}

DSINLINE void CSkinButton::ShowWindow(bool show)
{
    CHECKCALL(m_ptr->ShowWindow(bool2variant(show)));
}

DSINLINE void CSkinButton::GetControlRect(RECT* rect)
{
    CHECKCALL(m_ptr->GetControlRect(rect));
}

DSINLINE void CSkinButton::SetNormalSkin(int skinElement)
{
    CHECKCALL(m_ptr->SetNormalSkin(skinElement));
}

DSINLINE int CSkinButton::GetNormalSkin() const
{
    int retVal;
    CHECKCALL(m_ptr->GetNormalSkin(&retVal));
    return retVal;
}

DSINLINE void CSkinButton::SetCheckedSkin(int skinElement)
{
    CHECKCALL(m_ptr->SetCheckedSkin(skinElement));
}

DSINLINE int CSkinButton::GetCheckedSkin() const
{
    int retVal;
    CHECKCALL(m_ptr->GetCheckedSkin(&retVal));
    return retVal;
}

DSINLINE bool CSkinButton::IsChecked() const
{
    VARIANT_BOOL retVal;
    CHECKCALL(m_ptr->IsChecked(&retVal));
    return variant2bool(retVal);
}

DSINLINE void CSkinButton::SetCheck(bool check)
{
    CHECKCALL(m_ptr->SetCheck(bool2variant(check)));
}

DSINLINE int CSkinButton::GetCommandId() const
{
    int retVal;
    CHECKCALL(m_ptr->GetCommandId(&retVal));
    return retVal;
}

DSINLINE SIZE CSkinButton::GetSize() const
{
    SIZE retVal;
    CHECKCALL(m_ptr->GetSize(&retVal));
    return retVal;
}

DSINLINE void CSkinButton::SetStyle(const int style)
{
    CHECKCALL(m_ptr->SetStyle(style));
}

DSINLINE void CSkinButton::SetCommandId(const int commandId)
{
    CHECKCALL(m_ptr->SetCommandId(commandId));
}


DSINLINE CSkinProgressBar::CSkinProgressBar()
:m_ptr(0)
{
}

DSINLINE CSkinProgressBar::CSkinProgressBar(CComPtr<ISkinProgressBar> ptr)
:m_ptr(ptr)
{
}

DSINLINE void CSkinProgressBar::Release()
{
    m_ptr.Release();
}
	
DSINLINE bool CSkinProgressBar::Init(
	CSkinManager skinManager, 
    int skin,
    int id)
{
    return SUCCEEDED(m_ptr->Init2(
        skinManager.m_ptr,
        skin,
        id));
}
	
DSINLINE int CSkinProgressBar::GetId() const
{
    int retVal;
    CHECKCALL(m_ptr->GetId(&retVal));
    return retVal;
}

DSINLINE bool CSkinProgressBar::Create(HWND hwndParent, bool visible)
{
    return SUCCEEDED(m_ptr->Create((int)hwndParent,bool2variant(visible)));
}

DSINLINE HWND CSkinProgressBar::GetHwnd()
{
    int retVal;
    CHECKCALL(m_ptr->GetHwnd(&retVal));
    return (HWND)retVal;
}

DSINLINE void CSkinProgressBar::Close()
{
    CHECKCALL(m_ptr->Close());
}

DSINLINE void CSkinProgressBar::MoveWindow(int x, int y, int width, int height)
{
    CHECKCALL(m_ptr->MoveWindow(x, y, width, height));
}

DSINLINE void CSkinProgressBar::ShowWindow(bool show)
{
    CHECKCALL(m_ptr->ShowWindow(bool2variant(show)));
}

DSINLINE void CSkinProgressBar::GetControlRect(RECT* rect)
{
    CHECKCALL(m_ptr->GetControlRect(rect));
}

DSINLINE void CSkinProgressBar::SetSkinElement(int skinElement)
{
    CHECKCALL(m_ptr->SetSkinElement(skinElement));
}

DSINLINE int CSkinProgressBar::GetSkinElement() const
{
    int retVal;
    CHECKCALL(m_ptr->GetSkinElement(&retVal));
    return retVal;
}

DSINLINE void CSkinProgressBar::SetValue(int value)
{
    CHECKCALL(m_ptr->SetValue(value));
}

DSINLINE void CSkinProgressBar::SetIdentifier(const int identifier)
{
    CHECKCALL(m_ptr->SetIdentifier(identifier));
}

DSINLINE CSkinSlider::CSkinSlider()
:m_ptr(0)
{
}

DSINLINE CSkinSlider::CSkinSlider(CComPtr<ISkinSlider> ptr)
:m_ptr(ptr)
{
}

DSINLINE void CSkinSlider::Release()
{
    m_ptr.Release();
}

	
DSINLINE bool CSkinSlider::Init(
	CSkinManager skinManager, 
    ISkinSliderParent* parent,
    int id)
{
    return SUCCEEDED(m_ptr->Init2(
        skinManager.m_ptr,
        parent,
        id));
}
	
DSINLINE bool CSkinSlider::Create(HWND hwndParent, bool visible)
{
    return SUCCEEDED(m_ptr->Create((int)hwndParent,bool2variant(visible)));
}

DSINLINE HWND CSkinSlider::GetHwnd()
{
    int retVal;
    CHECKCALL(m_ptr->GetHwnd(&retVal));
    return (HWND)retVal;
}

DSINLINE void CSkinSlider::Close()
{
    CHECKCALL(m_ptr->Close());
}

DSINLINE void CSkinSlider::MoveWindow(int x, int y, int width, int height)
{
    CHECKCALL(m_ptr->MoveWindow(x, y, width, height));
}

DSINLINE void CSkinSlider::ShowWindow(bool show)
{
    CHECKCALL(m_ptr->ShowWindow(bool2variant(show)));
}

DSINLINE void CSkinSlider::GetControlRect(RECT* rect)
{
    CHECKCALL(m_ptr->GetControlRect(rect));
}


DSINLINE void CSkinSlider::SetValue(int value)
{
    CHECKCALL(m_ptr->SetValue(value));
}

DSINLINE void CSkinSlider::SetRange(int iMin, int iMax)
{
    CHECKCALL(m_ptr->SetRange(iMin, iMax));
}

DSINLINE void CSkinSlider::SetIdentifier(const int identifier)
{
    CHECKCALL(m_ptr->SetIdentifier(identifier));
}


DSINLINE STDMETHODIMP CBarChartParent::OnDrawBackground(IBarChart* ptr, IGraphics* graphics)
{
    CBarChart chart(ptr);
    OnDrawBackground(chart,CGraphics(graphics));
    return S_OK;
}

DSINLINE STDMETHODIMP CBarChartParent::OnClick(IBarChart* ptr, VARIANT_BOOL dbclk)
{
    CBarChart chart(ptr);
    OnClick(chart,variant2bool(dbclk));
    return S_OK;
}

DSINLINE STDMETHODIMP CBarChartParent::OnMouseHover(IBarChart* ptr)
{
    CBarChart chart(ptr);
    OnMouseHover(chart);
    return S_OK;
}

DSINLINE STDMETHODIMP CBarChartParent::OnMouseLeave(IBarChart* ptr)
{
    CBarChart chart(ptr);
    OnMouseLeave(chart);
    return S_OK;
}

DSINLINE STDMETHODIMP CBarChartParent::OnShowDetails(IBarChart* ptr)
{
    CBarChart chart(ptr);
    OnShowDetails(chart);
    return S_OK;
}

DSINLINE CBarChart::CBarChart()
:m_ptr(0)
{
}

DSINLINE CBarChart::CBarChart(CComPtr<IBarChart> ptr)
:m_ptr(ptr)
{
}
	
DSINLINE void CBarChart::Release()
{
    m_ptr.Release();
}

DSINLINE bool CBarChart::Init(
	CGlobalSettings settings, 
	CSkinManager skinManager, 
	IBarChartParent* parent,
    const std::string& label,
    int id)
{
    return SUCCEEDED(m_ptr->Init2(
	    settings.m_ptr, 
		skinManager.m_ptr, 
		parent,
        CComBSTR(label.c_str()),
        id));
}
DSINLINE bool CBarChart::Create(HWND hwndParent, bool visible)
{
    return SUCCEEDED(m_ptr->Create((int)hwndParent,bool2variant(visible)));
}

DSINLINE HWND CBarChart::GetHwnd()
{
    int retVal;
    CHECKCALL(m_ptr->GetHwnd(&retVal));
    return (HWND)retVal;
}

DSINLINE void CBarChart::Close()
{
    CHECKCALL(m_ptr->Close());
}

DSINLINE void CBarChart::MoveWindow(int x, int y, int width, int height)
{
    CHECKCALL(m_ptr->MoveWindow(x, y, width, height));
}

DSINLINE void CBarChart::ShowWindow(bool show)
{
    CHECKCALL(m_ptr->ShowWindow(bool2variant(show)));
}

DSINLINE void CBarChart::GetControlRect(RECT* rect)
{
    CHECKCALL(m_ptr->GetControlRect(rect));
}

DSINLINE void CBarChart::SetValue(int value)
{
    CHECKCALL(m_ptr->SetValue(value));
}

DSINLINE void CBarChart::SetType(int value)
{
    CHECKCALL(m_ptr->SetType(value));
}

DSINLINE void CBarChart::SetValueD(double value)
{
    CHECKCALL(m_ptr->SetValueD(value));
}

DSINLINE void CBarChart::SetRangeD(double min,double max)
{
    CHECKCALL(m_ptr->SetRangeD(min, max));
}

DSINLINE void CBarChart::SetLabel(BSTR label)
{
    CHECKCALL(m_ptr->SetLabel(label));
}

DSINLINE void CBarChart::SetLabelA(const std::string& label)
{
    CHECKCALL(m_ptr->SetLabel(CComBSTR(label.c_str())));
}

DSINLINE void CBarChart::GetFitSize(BSTR text, SIZE* sz)
{
    CHECKCALL(m_ptr->GetFitSize(text, sz));
}

DSINLINE void CBarChart::SetTextPosition(int position)
{
    CHECKCALL(m_ptr->SetTextPosition(position));
}

DSINLINE void CBarChart::SetChartSize(SIZE size)
{
    CHECKCALL(m_ptr->SetChartSize(size));
}

DSINLINE void CBarChart::SetValueType(int showValue, BSTR suffix)
{
    CHECKCALL(m_ptr->SetValueType(showValue, suffix));
}

DSINLINE void CBarChart::SetTextColor(DWORD argb)
{
    CHECKCALL(m_ptr->SetTextColor(argb));
}

DSINLINE void CBarChart::SetIdentifier(const int identifier)
{
    CHECKCALL(m_ptr->SetIdentifier(identifier));
}

DSINLINE STDMETHODIMP CDetailsWndParent::OnCreateDetailsWnd(IUnknown* unknown)
{
    CComQIPtr<IDetailsWnd,&IID_IDetailsWnd> ptr(unknown);
    CDetailsWnd details(ptr);
    OnCreateDetailsWnd(details);
    return S_OK;
}

DSINLINE STDMETHODIMP CDetailsWndParent::OnCloseDetailsWnd(IUnknown* unknown)
{
    CComQIPtr<IDetailsWnd,&IID_IDetailsWnd> ptr(unknown);
    CDetailsWnd details(ptr);
    OnCloseDetailsWnd(details);
    return S_OK;
}

DSINLINE CDetailsWnd::CDetailsWnd()
:m_ptr(0)
{
}

DSINLINE CDetailsWnd::CDetailsWnd(CComPtr<IDetailsWnd> ptr)
:m_ptr(ptr)
{
}

DSINLINE CDetailsWnd::~CDetailsWnd()
{
    Release();
}

DSINLINE void CDetailsWnd::Release()
{
    m_ptr.Release();
}

DSINLINE bool CDetailsWnd::Init(
	CGlobalSettings settings, 
	CSkinManager skinManager, 
    CPanelConfig panelConfig,
	IDetailsWndParent* parent, 
    const std::string& caption)
{
    return SUCCEEDED(m_ptr->Init(
	    settings.m_ptr, 
	    skinManager.m_ptr, 
        panelConfig.m_ptr,
	    parent, 
        CComBSTR(caption.c_str())));
}

DSINLINE bool CDetailsWnd::Init(
	CGlobalSettings settings, 
	CSkinManager skinManager, 
	IDetailsWndParent* parent, 
    const std::string& caption)
{
    return SUCCEEDED(m_ptr->Init(
	    settings.m_ptr, 
	    skinManager.m_ptr, 
        0,
	    parent, 
        CComBSTR(caption.c_str())));
}

DSINLINE void CDetailsWnd::RegisterParent(IDetailsWndParent* parent)
{
    CHECKCALL(m_ptr->RegisterParent(parent));
}

DSINLINE bool CDetailsWnd::Create(
    HWND hwndParent, 
    const POINT& origin, 
    SIZE size)
{
    return SUCCEEDED(m_ptr->Create((int)hwndParent,0,&origin,&size));
}

DSINLINE bool CDetailsWnd::Create(
    HWND hwndParent, 
    const POINT& origin)
{
    return SUCCEEDED(m_ptr->Create((int)hwndParent,0,&origin,0));
}

DSINLINE bool CDetailsWnd::Create(
    HWND hwndParent, 
    SIZE size)
{
    return SUCCEEDED(m_ptr->Create((int)hwndParent,0,0,&size));
}

DSINLINE bool CDetailsWnd::Create(HWND hwndParent)
{
    return SUCCEEDED(m_ptr->Create((int)hwndParent,0,0,0));
}

DSINLINE bool CDetailsWnd::Create(
    HWND hwndParent, 
    const RECT& parentArea, 
    SIZE size)
{
    POINT origin;
    CHECKCALL(m_ptr->AreaToOrigin(parentArea,&origin));
    return SUCCEEDED(m_ptr->Create((int)hwndParent,0,&origin,&size));
}

DSINLINE bool CDetailsWnd::Create(
    HWND hwndParent, 
    const RECT& parentArea)
{
    POINT origin;
    CHECKCALL(m_ptr->AreaToOrigin(parentArea,&origin));
    return SUCCEEDED(m_ptr->Create((int)hwndParent,0,&origin,0));
}

DSINLINE HWND CDetailsWnd::GetHwnd()
{
    int retVal;
    CHECKCALL(m_ptr->GetHwnd(&retVal));
    return (HWND)retVal;
}

DSINLINE SIZE CDetailsWnd::GetSize()
{
    SIZE retVal;
    CHECKCALL(m_ptr->GetSize(&retVal));
    return retVal;
}

DSINLINE void CDetailsWnd::SetSize(SIZE size, bool redraw)
{
    CHECKCALL(m_ptr->SetSize(size,bool2variant(redraw)));
}

DSINLINE void CDetailsWnd::SetCaption(const std::string& caption)
{
    CHECKCALL(m_ptr->SetCaption(CComBSTR(caption.c_str())));
}

DSINLINE RECT CDetailsWnd::GetInteriorRect()
{
    RECT retVal;
    CHECKCALL(m_ptr->GetInteriorRect(&retVal));
    return retVal;
}

DSINLINE Gdiplus::Color CDetailsWnd::GetBkColor()
{
    Gdiplus::ARGB argb;
    CHECKCALL(m_ptr->GetBkColor(&argb));
    return Gdiplus::Color(argb);
}

DSINLINE void CDetailsWnd::SetInterior(HWND hwnd)
{
    CHECKCALL(m_ptr->SetInterior((int)hwnd));
}

DSINLINE HWND CDetailsWnd::GetInterior()
{
    int retVal;
    CHECKCALL(m_ptr->GetInterior(&retVal));
    return (HWND)retVal;
}

DSINLINE void CDetailsWnd::Show()
{
    CHECKCALL(m_ptr->Show());
}

DSINLINE void CDetailsWnd::Fade()
{
    CHECKCALL(m_ptr->Fade());
}

DSINLINE void CDetailsWnd::Hide()
{
    CHECKCALL(m_ptr->Hide());
}

DSINLINE void CDetailsWnd::SetFixedSize(bool fixed)
{
    CHECKCALL(m_ptr->SetFixedSize(bool2variant(fixed)));
}

DSINLINE void CDetailsWnd::SetUserParam(long param)
{
    CHECKCALL(m_ptr->SetUserParam(param));
}

DSINLINE long CDetailsWnd::GetUserParam()
{
    long retVal;
    CHECKCALL(m_ptr->GetUserParam(&retVal));
    return retVal;
}

DSINLINE void CDetailsWnd::SetUserUnk(IUnknown* unk)
{
    CHECKCALL(m_ptr->SetUserUnk(unk));
}

DSINLINE void CDetailsWnd::GetUserUnk(IUnknown** unk)
{
    CHECKCALL(m_ptr->GetUserUnk(unk));
}

DSINLINE void CDetailsWnd::PinWindow()
{
    CHECKCALL(m_ptr->PinWindow());
}

DSINLINE void CDetailsWnd::UnpinWindow()
{
    CHECKCALL(m_ptr->UnpinWindow());
}

DSINLINE CTextDetailsWnd::CTextDetailsWnd(CComPtr<ITextDetailsWnd> ptr)
:CDetailsWnd(0),m_ptr(ptr)
{
    if (m_ptr)
    {
        m_ptr->QueryInterface(IID_IDetailsWnd,(void**)&(CDetailsWnd::m_ptr));
    }        
}

DSINLINE void CTextDetailsWnd::Release()
{
    CDetailsWnd::Release();
    m_ptr.Release();
}

DSINLINE void CTextDetailsWnd::Set(const std::string& text)
{
    CHECKCALL(m_ptr->Set(CComBSTR(text.c_str())));
}

DSINLINE void CTextDetailsWnd::Append(const std::string& text)
{
    CHECKCALL(m_ptr->Append(CComBSTR(text.c_str())));
}

DSINLINE CLinkDetailsWnd::CLinkDetailsWnd(CComPtr<ILinkDetailsWnd> ptr)
:CDetailsWnd(0),m_ptr(ptr)
{
    if (m_ptr)
    {
        m_ptr->QueryInterface(IID_IDetailsWnd,(void**)&(CDetailsWnd::m_ptr));
    }
}

DSINLINE void CLinkDetailsWnd::Release()
{
    CDetailsWnd::Release();
    m_ptr.Release();
}

DSINLINE void CLinkDetailsWnd::Set(const std::string& text)
{
    CHECKCALL(m_ptr->Set(CComBSTR(text.c_str())));
}

DSINLINE CHTMLDetailsWnd::CHTMLDetailsWnd(CComPtr<IHTMLDetailsWnd> ptr)
:CDetailsWnd(0),m_ptr(ptr)
{
    if (m_ptr)
    {
        m_ptr->QueryInterface(IID_IDetailsWnd,(void**)&(CDetailsWnd::m_ptr));
    }
}

DSINLINE void CHTMLDetailsWnd::Release()
{
    CDetailsWnd::Release();
    m_ptr.Release();
}

DSINLINE void CHTMLDetailsWnd::Set(const std::wstring& body)
{
    CHECKCALL(m_ptr->Set(CComBSTR(body.c_str())));
}

DSINLINE void CHTMLDetailsWnd::SetExternal(IDispatch* ext)
{
    CHECKCALL(m_ptr->SetExternal(ext));
}

DSINLINE void CHTMLDetailsWnd::ShowImages(bool showImages)
{
    CHECKCALL(m_ptr->ShowImages(bool2variant(showImages)));
}

DSINLINE CWebBrowserDetailsWnd::CWebBrowserDetailsWnd(CComPtr<IWebBrowserDetailsWnd> ptr)
:CDetailsWnd(0),m_ptr(ptr)
{
    if (m_ptr)
    {
        m_ptr->QueryInterface(IID_IDetailsWnd,(void**)&(CDetailsWnd::m_ptr));
    }
}

DSINLINE void CWebBrowserDetailsWnd::Release()
{
    CDetailsWnd::Release();
    m_ptr.Release();
}

DSINLINE void CWebBrowserDetailsWnd::Set(const std::string& url)
{
    CHECKCALL(m_ptr->Set(CComBSTR(url.c_str())));
}

DSINLINE void CWebBrowserDetailsWnd::ShowImages(bool showImages)
{
    CHECKCALL(m_ptr->ShowImages(bool2variant(showImages)));
}

//-----------------------------------------------------------------------------
// CEventHandler routines
//-----------------------------------------------------------------------------

DSINLINE CEventHandler::CEventHandler()
{
}

DSINLINE CEventHandler::CEventHandler(CComPtr<IEventHandler> ptr):m_ptr(ptr)
{
}

DSINLINE void CEventHandler::Init(CCanvas canvas,CComPtr<IDispatch> script)
{
    CHECKCALL(m_ptr->Init(canvas.m_ptr,script));
}

//-----------------------------------------------------------------------------
// CControlCreator routines
//-----------------------------------------------------------------------------

DSINLINE CControlCreator::CControlCreator()
{
}

DSINLINE CControlCreator::CControlCreator(CComPtr<IControlCreator> ptr):m_ptr(ptr)
{
}

DSINLINE CControl CControlCreator::CreateControl(
    const std::string& ctrlClass,
    CXmlNode description,
    CCanvas canvas)
{
    CControl control;
    CHECKCALL(
        m_ptr->CreateControl(
            CComBSTR(ctrlClass.c_str()),
            description.m_ptr,
            canvas.m_ptr,
            &control.m_ptr));
    return control;
}

DSINLINE CEventHandler CControlCreator::CreateEventHandler(const std::string& ctrlClass)
{
    CComPtr<IEventHandler> eventHandler;
    CHECKCALL(m_ptr->CreateEventHandler(CComBSTR(ctrlClass.c_str()),&eventHandler));
    return CEventHandler(eventHandler);
}



DSINLINE CControlFactory::CControlFactory(CComPtr<IControlFactory> ptr)
:m_ptr(ptr)
{
}

DSINLINE CTextOutput CControlFactory::CreateTextOutput()
{
    CComPtr<ITextOutput> retVal;
    CHECKCALL(m_ptr->CreateTextOutput(&retVal));
    return CTextOutput(retVal);
}

DSINLINE CListOutput CControlFactory::CreateListOutput()
{
    CComPtr<IListOutput> retVal;
    CHECKCALL(m_ptr->CreateListOutput(&retVal));
    return CListOutput(retVal);
}

DSINLINE CMarqueeOutput CControlFactory::CreateMarqueeOutput()
{
    CComPtr<IMarqueeOutput> retVal;
    CHECKCALL(m_ptr->CreateMarqueeOutput(&retVal));
    return CMarqueeOutput(retVal);
}

DSINLINE CSkinButton CControlFactory::CreateSkinButton()
{
    CComPtr<ISkinButton> retVal;
    CHECKCALL(m_ptr->CreateSkinButton(&retVal));
    return CSkinButton(retVal);
}

DSINLINE CSkinProgressBar CControlFactory::CreateSkinProgressBar()
{
    CComPtr<ISkinProgressBar> retVal;
    CHECKCALL(m_ptr->CreateSkinProgressBar(&retVal));
    return CSkinProgressBar(retVal);
}

DSINLINE CSkinSlider CControlFactory::CreateSkinSlider()
{
    CComPtr<ISkinSlider> retVal;
    CHECKCALL(m_ptr->CreateSkinSlider(&retVal));
    return CSkinSlider(retVal);
}

DSINLINE CBarChart CControlFactory::CreateBarChart()
{
    CComPtr<IBarChart> retVal;
    CHECKCALL(m_ptr->CreateBarChart(&retVal));
    return CBarChart(retVal);
}

DSINLINE CDetailsWnd CControlFactory::CreateDetailsWnd()
{
    CComPtr<IDetailsWnd> retVal;
    m_ptr->CreateDetailsWnd(&retVal);
    return CDetailsWnd(retVal);
}

DSINLINE CTextDetailsWnd CControlFactory::CreateTextDetailsWnd()
{
    CComPtr<ITextDetailsWnd> retVal;
    m_ptr->CreateTextDetailsWnd(&retVal);
    return CTextDetailsWnd(retVal);
}

DSINLINE CLinkDetailsWnd CControlFactory::CreateLinkDetailsWnd()
{
    CComPtr<ILinkDetailsWnd> retVal;
    m_ptr->CreateLinkDetailsWnd(&retVal);
    return CLinkDetailsWnd(retVal);
}

DSINLINE CHTMLDetailsWnd CControlFactory::CreateHTMLDetailsWnd()
{
    CComPtr<IHTMLDetailsWnd> retVal;
    m_ptr->CreateHTMLDetailsWnd(&retVal);
    return CHTMLDetailsWnd(retVal);
}

DSINLINE CWebBrowserDetailsWnd CControlFactory::CreateWebBrowserDetailsWnd()
{
    CComPtr<IWebBrowserDetailsWnd> retVal;
    m_ptr->CreateWebBrowserDetailsWnd(&retVal);
    return CWebBrowserDetailsWnd(retVal);
}

DSINLINE SIZE CControlFactory::InteriorToDetailsSize(
    const SIZE& interior, 
    const std::string& caption)
{
    SIZE retVal;
    CHECKCALL(m_ptr->InteriorToDetailsSize(
        interior, 
        CComBSTR(caption.c_str()),
        &retVal));
    return retVal;
}

DSINLINE void CControlFactory::HideAllDetails()
{
    CHECKCALL(m_ptr->HideAllDetails());
}

DSINLINE bool CControlFactory::IsDetailsDisplayed()
{
    VARIANT_BOOL retVal;
    CHECKCALL(m_ptr->IsDetailsDisplayed(&retVal));
    return variant2bool(retVal);
}

DSINLINE void CControlFactory::RegisterCreator(
    const std::string& ctrlClass, 
    CControlCreator creator)
{
    m_ptr->RegisterCreator(CComBSTR(ctrlClass.c_str()),creator.m_ptr);
}

DSINLINE void CControlFactory::UnregisterCreator(const std::string& ctrlClass)
{
    m_ptr->UnregisterCreator(CComBSTR(ctrlClass.c_str()));
}

DSINLINE CControl CControlFactory::CreateControl(
    const std::string& ctrlClass,
    CXmlNode description,
    CCanvas canvas)
{
    CComPtr<IControl> control;
    CHECKCALL(
        m_ptr->CreateControl(
            CComBSTR(ctrlClass.c_str()),
            description.m_ptr,
            canvas.m_ptr,
            &control));
    return CControl(control);
}

DSINLINE CEventHandler CControlFactory::CreateEventHandler(const std::string& ctrlClass)
{
    CComPtr<IEventHandler> eventHandler;
    CHECKCALL(m_ptr->CreateEventHandler(CComBSTR(ctrlClass.c_str()),&eventHandler));
    return CEventHandler(eventHandler);
}


DSINLINE CEventHandlersManager::CEventHandlersManager()
{
}

DSINLINE CEventHandlersManager::CEventHandlersManager(CComPtr<IEventHandlersManager> ptr):m_ptr(ptr)
{
}

DSINLINE void CEventHandlersManager::Init(
    CControlFactory controlFactory,
    CCanvas canvas,
    CComPtr<IDispatch> script)
{
    CHECKCALL(m_ptr->Init(controlFactory.m_ptr,canvas.m_ptr,script));
}

DSINLINE CEventHandler CEventHandlersManager::GetEventHandler(const std::string& ctrlClass)
{
    CComPtr<IEventHandler> eventHandler;
    CHECKCALL(m_ptr->GetEventHandler(CComBSTR(ctrlClass.c_str()),&eventHandler));
    return CEventHandler(eventHandler);
}


DSINLINE CSidebar::CSidebar()
    :m_ptr(0)
{
}

DSINLINE CSidebar::CSidebar(CComPtr<ISidebar> ptr)
    :m_ptr(ptr)
{
}

DSINLINE CTranslator CSidebar::GetTranslator() const
{
	CComPtr<ITranslator> retVal;
	CHECKCALL(m_ptr->GetTranslator(&retVal));
	return retVal;
}

DSINLINE CSkinManager CSidebar::GetSkinManager() const
{
	CComPtr<ISkinManager> retVal;
	CHECKCALL(m_ptr->GetSkinManager(&retVal));
	return retVal;
}

DSINLINE CAlertManager CSidebar::GetAlertManager() const
{
	CComPtr<IAlertManager> retVal;
	CHECKCALL(m_ptr->GetAlertManager(&retVal));
	return retVal;
}

DSINLINE CCommandManager CSidebar::GetCommandManager() const
{
	CComPtr<ICommandManager> retVal;
	CHECKCALL(m_ptr->GetCommandManager(&retVal));
	return retVal;
}

DSINLINE CGlobalSettings CSidebar::GetGlobalSettings() const
{
	CComPtr<IGlobalSettings> retVal;
	CHECKCALL(m_ptr->GetGlobalSettings(&retVal));
	return retVal;
}

DSINLINE CControlFactory CSidebar::GetControlFactory() const
{
	CComPtr<IControlFactory> retVal;
	CHECKCALL(m_ptr->GetControlFactory(&retVal));
	return retVal;
}

DSINLINE void CSidebar::SaveConfiguration()
{
    CHECKCALL(m_ptr->SaveConfiguration());
}

DSINLINE bool CSidebar::OpenXml(CXmlNode* node, const std::string& filePath, std::string* errorMsg) const
{
    CComBSTR msg;
    bool retVal=SUCCEEDED(m_ptr->OpenXml(CComBSTR(filePath.c_str()),&msg,&node->m_ptr));
    *errorMsg=bstr2str(msg);    
    return retVal;
}

DSINLINE CGraphics CSidebar::CreateGraphics(HDC hdc)
{
	CComPtr<IGraphics> retVal;
	CHECKCALL(m_ptr->CreateGraphics(int(hdc),&retVal));
	return CGraphics(retVal);
}

DSINLINE int CSidebar::GetVersion(int* major, int* minor) const
{
    int retVal;
	CHECKCALL(m_ptr->GetVersion(major,minor,&retVal));
    return retVal;
}
    
DSINLINE std::string CSidebar::GetSettingsDir() const
{
    CComBSTR dir;
    CHECKCALL(m_ptr->GetSettingsDir(&dir));
    return bstr2str(dir);    
}

DSINLINE void CSidebar::Log(const std::string& text)
{
	CHECKCALL(m_ptr->Log(CComBSTR(text.c_str())));
}

DSINLINE bool CSidebar::RegisterPanel(
        const std::string& panelClass,
        const std::string& panelCaption,
        const std::string& panelDescription,
        HIMAGELIST imagelist,
        const std::string& panelIcon,
        const std::string& panelCategory,
        const std::string& categoryIcon,
        const std::string& panelFiles,
        const std::string& panelCanvas,
        int pluginCookie)
{
	return SUCCEEDED(m_ptr->RegisterPanel(
        CComBSTR(panelClass.c_str()),
        CComBSTR(panelCaption.c_str()),
        CComBSTR(panelDescription.c_str()),
        (int)imagelist,
        CComBSTR(panelIcon.c_str()),
        CComBSTR(panelCategory.c_str()),
        CComBSTR(categoryIcon.c_str()),
        CComBSTR(panelFiles.c_str()),
        CComBSTR(panelCanvas.c_str()),
        pluginCookie));
}

DSINLINE bool CSidebar::RegisterPanel2(
        const std::string& panelClass,
        const std::string& panelCaption,
        const std::string& panelDescription,
        HIMAGELIST imagelist,
        const std::string& panelIcon,
        const std::string& panelCategory,
        const std::string& categoryIcon,
        const std::string& panelFiles,
        const std::string& panelCanvas,
        int panelFlags,
        int pluginCookie)
{
	return SUCCEEDED(m_ptr->RegisterPanel2(
        CComBSTR(panelClass.c_str()),
        CComBSTR(panelCaption.c_str()),
        CComBSTR(panelDescription.c_str()),
        (int)imagelist,
        CComBSTR(panelIcon.c_str()),
        CComBSTR(panelCategory.c_str()),
        CComBSTR(categoryIcon.c_str()),
        CComBSTR(panelFiles.c_str()),
        CComBSTR(panelCanvas.c_str()),
        panelFlags,
        pluginCookie));
}

DSINLINE void SetSkin(HWND sidebar, const char* path)
{
    COPYDATASTRUCT cds;
    cds.dwData=DSC_SETSKIN;
    cds.lpData=(LPVOID)path;
    cds.cbData=lstrlen(path)+1;
    SendMessage(sidebar,WM_COPYDATA,0,LPARAM(&cds));
}

DSINLINE void TestSkin(HWND sidebar, const char* path)
{
    COPYDATASTRUCT cds;
    cds.dwData=DSC_TESTSKIN;
    cds.lpData=(LPVOID)path;
    cds.cbData=lstrlen(path)+1;
    SendMessage(sidebar,WM_COPYDATA,0,LPARAM(&cds));
}

DSINLINE void HandleFile(HWND sidebar, const char* path)
{
    COPYDATASTRUCT cds;
    cds.dwData=DSC_HANDLEFILE;
    cds.lpData=(LPVOID)path;
    cds.cbData=lstrlen(path)+1;
    SendMessage(sidebar,WM_COPYDATA,0,LPARAM(&cds));
}

DSINLINE void ExecuteCommand(HWND sidebar, const char* command)
{
    COPYDATASTRUCT cds;
    cds.dwData=DSC_EXECUTECOMMAND;
    cds.lpData=(LPVOID)command;
    cds.cbData=lstrlen(command)+1;
    SendMessage(sidebar,WM_COPYDATA,0,LPARAM(&cds));
}

DSINLINE void ShowSidebar(HWND sidebar)
{
    COPYDATASTRUCT cds;
    cds.dwData=DSC_SHOWSIDEBAR;
    cds.lpData=NULL;
    cds.cbData=0;
    SendMessage(sidebar,WM_COPYDATA,0,LPARAM(&cds));
}

DSINLINE HWND GetUndockedPanel(HWND hwnd)
{
    while (GetWindowLong(hwnd,GWL_STYLE)&WS_CHILD)
    {
        hwnd=GetParent(hwnd);
    }
    return SendMessage(hwnd,RegisterWindowMessage(RWM_ISUNDOCKEDPANEL),0,0)?hwnd:0;
}

DSINLINE BOOL InvalidatePanel(HWND hwnd, CONST RECT *lpRect, BOOL bErase)
{
    if (HWND undocked=GetUndockedPanel(hwnd))
    {
        return ::SendMessage(undocked,WM_REDRAWUNDOCKED,WPARAM(hwnd),LPARAM(lpRect));
    }
    else
    {
        return ::InvalidateRect(hwnd,lpRect,bErase);
    }
}

DSINLINE ESkinState GetNormalSkin(HWND hwnd)
{
    return GetUndockedPanel(hwnd)?Undocked:Normal;
}

}

#endif
